#!/usr/bin/python

#~ TODO: 
#~ * support for not continous work (long breaks during work)

import wx
import glob
import wx.grid
import datetime
import time
import socket
import sys
import pickle
import ConfigParser
import exceptions

def makeTime(h,m,s):
    now=datetime.datetime.now()
    return datetime.datetime(now.year,now.month,now.day, h,m,s)
class Overtime:
    class Config:
        def __init__(self):            
            config = ConfigParser.RawConfigParser()
            if len(config.read(['overtime.cfg']))==0:
                dialog = wx.TextEntryDialog(None, 
                    "It seems this is the first time you run Overtime Meter.\n"
                    "Please enter below a host:port you have access to only at work.\n"
                    "It there is no such host then please select Cancel\n"
                    "(Your selection will be saved in overtime.cfg file. "
                    "Remove the file to reset configuration)",
                    "Setup", "10.2.0.10:80")
                if wx.ID_OK==dialog.ShowModal():
                    value = dialog.GetValue()
                    pair = value.split(':',2)
                    self.host = (pair[0], int(pair[1]))
                else:
                    self.host = None
                dialog.Destroy()
                self.interval = 5
                self.save(config)
            else:
                if config.has_section('HostAtWork'):
                    host = config.get('HostAtWork','Host')
                    port = config.getint('HostAtWork','Port')
                    self.host = (host,port)
                else:
                    self.host=None
                self.interval = config.getint('General','Interval')
            
        def save(self,config):
            if self.host:
                config.add_section('HostAtWork')  
                config.set('HostAtWork','Host',self.host[0])
                config.set('HostAtWork','Port',self.host[1])
            config.add_section('General')
            config.set('General','Interval',self.interval)
            config.write(open('overtime.cfg','w'))

    ##################################################################################    
    class Frame(wx.Frame):
        def __init__(self, meter):
            wx.Frame.__init__(self,None,-1,"Overtime Meter",wx.DefaultPosition)
            self.Show(False);
            self.taskIcon = wx.TaskBarIcon()
            self.offIcon = wx.Icon('icons/off.ico', wx.BITMAP_TYPE_ICO)
            self.onIcon = wx.Icon('icons/on.ico', wx.BITMAP_TYPE_ICO)
            self.overtimeIcon = wx.Icon('icons/overtime.ico', wx.BITMAP_TYPE_ICO)
            self.taskIcon.SetIcon(self.offIcon)  
            self.InitMenu()
            self.meter = meter
            self.grid = Overtime.HistoryGrid(self, self.meter)
            self.CLOSE_BUTTON=1201
            self.EXPORT_BUTTON=1202
            self.button = wx.Button(self,self.CLOSE_BUTTON,'Close')
            self.exportBtn = wx.Button(self, self.EXPORT_BUTTON, 'Export to CSV')
            self.boxSizer = wx.BoxSizer(orient=wx.VERTICAL)
            self.boxSizer.AddWindow(self.grid, 1, border=0, flag=wx.ALL | wx.GROW)
            self.btnSizer = wx.BoxSizer(orient=wx.HORIZONTAL)
            self.btnSizer.AddWindow(self.exportBtn, 0, border=5, flag=wx.ALL | wx.ALIGN_RIGHT)
            self.btnSizer.AddWindow(self.button, 0, border=5, flag=wx.ALL | wx.ALIGN_RIGHT)
            self.boxSizer.AddSizer(self.btnSizer, 0, flag=wx.ALL | wx.ALIGN_RIGHT)
            wx.EVT_BUTTON(self.button, self.CLOSE_BUTTON, lambda event: self.Show(False));
            wx.EVT_BUTTON(self.exportBtn, self.EXPORT_BUTTON, lambda event: self.ExportToCSV());
            self.SetSizer(self.boxSizer)        
    
        def InitMenu(self):
            self.MENU_CLOSE = 1001
            self.MENU_STARTED = 1003
            self.menu = wx.Menu()
            self.started = self.menu.Append(self.MENU_STARTED,"Working", "", wx.ITEM_CHECK)
            self.menu.Append(self.MENU_CLOSE,"Quit")
            wx.EVT_TASKBAR_LEFT_UP(self.taskIcon, lambda event: self.Toggle())
            wx.EVT_TASKBAR_RIGHT_UP(self.taskIcon, lambda event: self.taskIcon.PopupMenu(self.menu))
            wx.EVT_MENU(self.taskIcon, self.MENU_CLOSE, lambda event: sys.exit())
            wx.EVT_MENU(self.taskIcon, self.MENU_STARTED, lambda event: self.ToggleStarted())
            wx.EVT_CLOSE(self, self.OnClose)

        def Toggle(self):
            if not self.IsShown():
                self.grid.read()
                self.AdjustSize()
                self.Center();
            self.Show(not self.IsShown())

        def ExportToCSV(self):
            filters = 'CSV files (*.csv)|*.csv|All files (*.*)|*.*'
            dialog = wx.FileDialog(self, 
                wildcard = filters,
                style = wx.SAVE | wx.OVERWRITE_PROMPT)
            if wx.ID_OK == dialog.ShowModal():
                self.grid.ExportToCSV(dialog.GetPath())
            dialog.Destroy()
        def AdjustSize(self):
            size=self.GetAdjustedBestSize()
            size.SetWidth(size.GetWidth()+25);
            size.SetHeight(size.GetHeight()+25);            
            self.SetSize(size);
            size=self.GetAdjustedBestSize()
            self.SetSize(size);
            
        def OnClose(self,event):
            self.Show(False)
            return False
        
        def ToggleStarted(self):
            if self.started.IsChecked():
                self.meter.stop()
            else:
                self.meter.start()
            
        def SetTooltip(self,icon,tip):
            self.taskIcon.SetIcon(icon,tip)    
            
    ##################################################################################    
    class Meter(wx.App):
        FORMAT = "%H:%M,%S"
        
        def OnInit(self):
            TIMER_ID = wx.NewId()
            self.timer = wx.Timer(self, TIMER_ID)
            wx.EVT_TIMER(self, TIMER_ID, lambda event: self.OnTimer())
            self.frame = Overtime.Frame(self)
            self.SetTopWindow(self.frame)
            return True        

        def run(self):
            if self.isInWork():
                self.start()
            self.MainLoop()
                  
        def start(self):
            self.sessionStart = self.GetSessionStart()            
            self.SetTooltip(self.frame.offIcon, datetime.datetime.now())
            self.StartTimer()

        def stop(self):
            self.timer.Stop()
            self.SetTooltip(self.frame.offIcon, datetime.datetime.now())
            self.frame.started.Check(False)
        
        def isInWork(self):
            if self.config.host is None:                
                return True
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.settimeout(1.5)
            try:
                s.connect(self.config.host)
                s.close()
                return True
            except Exception:
                return False
                
        def GetDateFromFile(self,filename):
            log = open(filename)
            loaded = time.strptime(log.read(), self.FORMAT)            
            return makeTime(loaded[3],loaded[4],loaded[5])
            
        def GetSessionStart(self):
            session = datetime.date.today().strftime("%Y-%m-%d");
            session_start_log = "data/"+session+".start.log"
            self.session_stop_log = "data/"+session+".stop.log"
            try:
                return self.GetDateFromFile(session_start_log)
            except IOError:
                #theres no file today, lets create one
                logFile = file(session_start_log,"w")
                start = datetime.datetime.now()
                logFile.write(start.strftime(self.FORMAT))
                logFile.close()
                return start
                
        def StartTimer(self):
            self.timer.Start(self.config.interval * 1000)
            self.frame.started.Check(True)
            
        def SetTooltip(self,icon,now):
            working = makeTime(0,0,0) + (now - self.sessionStart)
            left=makeTime(8,0,0) - (now - self.sessionStart);
            self.frame.SetTooltip(icon, working.strftime(self.FORMAT) + " (to go: "+left.strftime(self.FORMAT)+")")

        def OnTimer(self):
            now=datetime.datetime.now()
            working = makeTime(0,0,0) + (now - self.sessionStart)
            if working > makeTime(8,0,0):
                icon = self.frame.overtimeIcon
            else:
                icon = self.frame.onIcon
            self.SetTooltip(icon, now)
            log = file(self.session_stop_log,"w");
            log.write(now.strftime(self.FORMAT)) 
            log.close()
    ############################################################################        
    class HistoryGrid(wx.grid.Grid):
        def __init__(self, parent, meter):        
            wx.grid.Grid.__init__(self, parent, -1)
            self.meter = meter
            self.CreateGrid(1, 5)
            self.DisableDragGridSize()
            self.DisableCellEditControl()
            #wx.grid.EVT_GRID_RANGE_SELECT( self, lambda evt: self.ClearSelection())
            self.SetColLabelValue(0,"Start")
            self.SetColLabelValue(1,"Stop")
            self.SetColLabelValue(2,"Working")
            self.SetColLabelValue(3,"Overtime")
            self.SetColLabelValue(4,"Overtime Sum")

        def ExportToCSV(self, filename):
            file=open(filename,'w')
            file.write('"Date"')
            for i in range(0, self.GetNumberCols()):
                file.write(',"'+self.GetColLabelValue(i)+'"')
            file.write('\n')
            for j in range(0,self.GetNumberRows()):
                file.write('"'+self.GetRowLabelValue(j)+'"')
                for i in range(0, self.GetNumberCols()):
                    file.write(',"'+self.GetCellValue(j,i)+'"')
                file.write('\n')
            file.close()
            
        def read(self):
            starts = glob.glob('./data/*.start.log')
            stops = glob.glob('./data/*.stop.log')
            self.DeleteRows(0,self.GetNumberRows())
            self.InsertRows(0,len(stops))
            overtimeSum = makeTime(0,0,0);
            for i in range(0,len(stops)):
                self.SetRowLabelValue(i,stops[i][7:-9])
                start=self.meter.GetDateFromFile(starts[i])
                stop =self.meter.GetDateFromFile(stops[i])
                self.SetCellValue(i,0,start.strftime(self.meter.FORMAT))
                self.SetCellValue(i,1,stop.strftime(self.meter.FORMAT))
                working = makeTime(0,0,0)+(stop-start)
                self.SetCellValue(i,2,working.strftime(self.meter.FORMAT))
                if(working > makeTime(8,0,0)):
                    overtime = makeTime(0,0,0)+(working - makeTime(8,0,0))
                    overtimeSum += (working - makeTime(8,0,0))
                    self.SetCellValue(i,3,overtime.strftime(self.meter.FORMAT))
                else:
                    overtimeSum -= (makeTime(8,0,0)-working)
                    overtime = makeTime(0,0,0)+(makeTime(8,0,0) - working)
                    self.SetCellValue(i,3,"-"+overtime.strftime(self.meter.FORMAT))
                for j in range(0,3):
                        self.SetReadOnly(i,j)
                if overtimeSum < makeTime(0,0,0):
                    displaySum = makeTime(0,0,0) + (makeTime(0,0,0) - overtimeSum)
                    self.SetCellValue(i,4, "-" + displaySum.strftime(self.meter.FORMAT))
                else:
                    self.SetCellValue(i,4, overtimeSum.strftime(self.meter.FORMAT))
            self.AutoSizeColumns();
            
#############################################################################
def run():
    meter = Overtime.Meter()
    meter.config = Overtime.Config()
    meter.run()
        
run()
